package Config::SqlConstants ;

=head1 NAME

Config::SqlConstants - Sql Constants

=head1 SYNOPSIS

use Config::SqlConstants ;

=head1 DESCRIPTION

MySql constants

=head1 AUTHOR

Steve Price 

=head1 BUGS

None that I know of!

=head1 INTERFACE


=cut


use strict ;

## Version ##################################################################
our $VERSION = '1.000' ;

## Class Variables ##########################################################
our $SQL_USER = 'steve' ;
our $SQL_PASSWORD = 'zzsqlsteve' ;


1;


